@@ -340,15 +340,10 @@ public abstract class BDHandler implements Player, ServiceContentHandler {
      * notifications from app
      */
 
-    protected void statusEvent(int event, int param) {
-        if (isClosed) return;
+    protected boolean statusEvent(int event, int param) {
+        if (isClosed) return false;
         commandQueue.put(new PlayerAction(this, PlayerAction.ACTION_STATUS, new Integer(event), param));
-    }
-
-    protected void rateChanged(float rate) {
-        if (isClosed) return;
-        PlayerAction action = new PlayerAction(this, PlayerAction.ACTION_RATE_CHANGED, new Float(rate));
-        commandQueue.put(action);
+        return true;
     }
 
     /*
@@ -461,6 +456,7 @@ public abstract class BDHandler implements Player, ServiceContentHandler {
         case Unrealized:
             state = Realizing;
             notifyListeners(new TransitionEvent(this, Unrealized, Realizing, Realized));
+            /* fall thru */
         case Realizing:
             ControllerErrorEvent error = doRealize();
             if (error == null) {
@@ -484,9 +480,11 @@ public abstract class BDHandler implements Player, ServiceContentHandler {
         case Realizing:
             if (!doRealizeAction())
                 return false;
+            /* fall thru */
         case Realized:
             state = Prefetching;
             notifyListeners(new TransitionEvent(this, Realized, Prefetching, Prefetched));
+            /* fall thru */
         case Prefetching:
 
             if (!PlayerManager.getInstance().allocateResource(this)) {
@@ -515,10 +513,12 @@ public abstract class BDHandler implements Player, ServiceContentHandler {
         case Realizing:
             if (!doRealizeAction())
                 return false;
+            /* fall thru */
         case Realized:
         case Prefetching:
             if (!doPrefetchAction())
                 return false;
+            /* fall thru */
         case Prefetched:
             ControllerErrorEvent error = doStart(at);
             if (error == null) {
@@ -593,7 +593,7 @@ public abstract class BDHandler implements Player, ServiceContentHandler {
         PlayerManager.getInstance().unregisterPlayer(this);
     }
 
-    private class PlayerAction extends BDJAction {
+    private static class PlayerAction extends BDJAction {
         private PlayerAction(BDHandler player, int action, Object param) {
             this(player, action, param, -1);
         }
@@ -635,9 +635,6 @@ public abstract class BDHandler implements Player, ServiceContentHandler {
                 player.doSetRate((Float)param);
                 break;
 
-            case ACTION_RATE_CHANGED:
-                player.doRateChanged(((Float)param).floatValue());
-                break;
             case ACTION_STATUS:
                 switch (((Integer)param).intValue()) {
                 case Libbluray.BDJ_EVENT_CHAPTER:
@@ -676,6 +673,13 @@ public abstract class BDHandler implements Player, ServiceContentHandler {
                 case Libbluray.BDJ_EVENT_SEEK:
                     player.doSeekNotify(param2 * 2 /* 45kHz -> 90kHz */);
                     break;
+                case Libbluray.BDJ_EVENT_RATE:
+                    float rate = (float)param2 / 90000.0f;
+                    if (rate < 0.0f) rate = -rate;
+                    if (rate < 0.01f) rate = 0.0f;
+                    if (rate > 0.99f && rate < 1.01f) rate = 1.0f;
+                    player.doRateChanged(rate);
+                    break;
                 default:
                     System.err.println("Unknown ACTION_STATUS: id " + param + ", value " + param2);
                     break;
@@ -704,7 +708,6 @@ public abstract class BDHandler implements Player, ServiceContentHandler {
         public static final int ACTION_SET_RATE = 9;
 
         public static final int ACTION_STATUS = 10;
-        public static final int ACTION_RATE_CHANGED = 11;
     }
 
     protected int state = Unrealized;
