@@ -30,16 +30,16 @@ import org.videolan.StreamInfo;
 
 public class VideoControl implements VideoPresentationControl {
     protected VideoControl(Handler player) {
-        this.player = player;
+        //this.player = player;
     }
 
     protected HScreenRectangle getNormalizedRectangle(Dimension dimension, Rectangle rectangle) {
         if ((dimension.width == 0) || (dimension.height == 0))
             return new HScreenRectangle(0, 0, 0, 0);
-        float x = rectangle.x / dimension.width;
-        float y = rectangle.y / dimension.height;
-        float w = rectangle.width / dimension.width;
-        float h = rectangle.height / dimension.height;
+        float x = (float)rectangle.x / dimension.width;
+        float y = (float)rectangle.y / dimension.height;
+        float w = (float)rectangle.width / dimension.width;
+        float h = (float)rectangle.height / dimension.height;
         return new HScreenRectangle(x, y, w, h);
     }
 
@@ -146,7 +146,7 @@ public class VideoControl implements VideoPresentationControl {
         throw new Error("Not implemented"); // TODO implement
     }
 
-    private Handler player;
+    //private Handler player;
     private HScreenRectangle srcArea = new HScreenRectangle(0.0f, 0.0f, 1.0f, 1.0f);
     private HScreenRectangle dstArea = new HScreenRectangle(0.0f, 0.0f, 1.0f, 1.0f);
 }
